"use strict";
const ApiConnection_1 = require("./ApiConnection");
const TextMessageSensitive_1 = require("../../utils/sensitive/TextMessageSensitive");
const Message_1 = require("../../models/message/Message");
const TextMessage_1 = require("../../models/message/TextMessage");
const SendMessageRequest_1 = require("../../models/clientMessages/SendMessageRequest");
/**
 * Interface for Developer to develop a NasimBot
 * Created by amin on 9/3/16.
 * Modified by emran.
 */
class NasimBot {
    /**
     *
     * @param botToken
     */
    constructor(botToken) {
        this._botToken = botToken;
        this._apiConnection = new ApiConnection_1.ApiConnection(botToken);
        this._hearsPacks = [];
        this._conversations = [];
        this.setOnMessage();
    }
    /**
     * Hears for a list of keywords or a {@link Sensitive Sensitive}
     * @param keywords
     * @param callback
     */
    hears(keywords, callback) {
        let sensitive = Array.isArray(keywords) ? new TextMessageSensitive_1.TextMessageSensitive(keywords) : keywords;
        this._hearsPacks.push(new HearsPack(sensitive, callback, this));
    }
    setConversation(conversation) {
        this._conversations.push(conversation);
    }
    /**
     * Sets a default callback to get called when no subscribed conversation or hears, matches with a message.
     * @param message A message to send as a default response.
     */
    setDefaultCallback(callback) {
        this._defaultCallback = callback;
    }
    /**
     * received message that user send for bot and check equal with which commands and call appropriate command callback
     */
    setOnMessage() {
        this._apiConnection.setOnMessage((message, sender) => {
            //Priority order of passing message:
            //
            //First priority: active conversations.
            //Second: start a new conversation
            //Third: check for hears
            //Search for active conversations if any.
            let activeConv = null;
            for (let conv of this._conversations) {
                if (conv.isActive(sender)) {
                    activeConv = conv;
                    break;
                }
            }
            if (activeConv != null) {
                //An active bot is already unfinished. So pass the message to it to handle.
                activeConv.handleMessage(this, message, sender);
            }
            else {
                //No active conversation. => Priority 2:
                let anyConvStarted = false;
                for (let conv of this._conversations) {
                    if (conv.canStart(message, sender)) {
                        conv.handleMessage(this, message, sender);
                        anyConvStarted = true;
                        break;
                    }
                }
                if (!anyConvStarted) {
                    //No conversation matched the message. => Priority 3.
                    let anyHearsMatched = false;
                    for (let sensitivePack of this._hearsPacks) {
                        if (sensitivePack.callIfMatches(message, sender)) {
                            anyHearsMatched = true;
                            break;
                        }
                    }
                    if (!anyHearsMatched) {
                        if (this._defaultCallback != null) {
                            //Default action
                            this._defaultCallback(message, new Responder(this, sender));
                        }
                    }
                }
            }
        });
    }
    /**
     * Sends a message to the given user.
     * @param message send message with apiConnection.send
     * @return {Promise<Response>}
     */
    send(message, receiver) {
        return this.sendRequest(new SendMessageRequest_1.SendMessageRequest(message, receiver));
    }
    /**
     * Sends a request to the server.
     * @param request
     * @return {Promise<Response>}
     */
    sendRequest(request) {
        return this._apiConnection.send(request);
    }
    /**
     * return botToken
     * @return {string}
     */
    get botToken() {
        return this._botToken;
    }
}
exports.NasimBot = NasimBot;
/**
 * implement replying to user
 */
class Responder {
    /**
     *
     * @param bot
     * @param peer
     */
    constructor(bot, peer) {
        this.bot = bot;
        this._peer = peer;
    }
    /**
     * send message to peerUser
     * @param message
     */
    reply(message) {
        let msg;
        if (message instanceof Message_1.Message)
            msg = message;
        else
            msg = new TextMessage_1.TextMessage(message);
        return this.bot.send(msg, this._peer);
    }
    get peer() {
        return this._peer;
    }
}
exports.Responder = Responder;
/**
 * Created by emran on 9/15/16
 */
class HearsPack {
    constructor(sensitive, callback, bot) {
        this._sensitive = sensitive;
        this._callback = callback;
        this.bot = bot;
    }
    get sensitive() {
        return this._sensitive;
    }
    get callback() {
        return this._callback;
    }
    /**
     * Calls the proper callback if the corresponding sensitive matches.
     * @param message The message to check if matches.
     * @return {boolean} True if the callback called; false otherwise.
     */
    callIfMatches(message, sender) {
        let result = this.sensitive.match(message);
        if (result)
            this.callback.call(this, message, new Responder(this.bot, sender));
        return result;
    }
}
exports.HearsPack = HearsPack;
//# sourceMappingURL=NasimBot.js.map